@@ -6,7 +6,7 @@
 #' @name plot_prior
 #'
 #' @param prior_sd Scale of the prior distribution, approximately the standard deviation
-#'                 of a beta distribution (defaults to sqrt(1/12)).
+#'                 of a beta distribution (defaults to 0.5).
 #'
 #' @param iter Number of iterations (prior samples; defaults to 5000).
 #'
@@ -17,7 +17,7 @@
 #'
 #' plot_prior(prior_sd = 0.25, iter = 250)
 #' @export
-plot_prior <- function(prior_sd = sqrt(1/12), iter = 5000){
+plot_prior <- function(prior_sd = 0.5, iter = 5000){
 
 
   # matrix dimensions for prior
